@@ -3,6 +3,7 @@ from ._rust_ephem import (  # type: ignore[import-untyped]
     ConstraintResult,
     ConstraintViolation,
     GroundEphemeris,
+    OEMEphemeris,
     SPICEEphemeris,
     TLEEphemeris,
     VisibilityWindow,
@@ -29,6 +30,7 @@ from .constraints import (
     NotConstraint,
     OrConstraint,
     SunConstraint,
+    XorConstraint,
 )
 
 __all__ = [
@@ -41,9 +43,11 @@ __all__ = [
     "CombinedConstraintConfig",
     "AndConstraint",
     "OrConstraint",
+    "XorConstraint",
     "NotConstraint",
     "TLEEphemeris",
     "SPICEEphemeris",
+    "OEMEphemeris",
     "GroundEphemeris",
     "Constraint",
     "ConstraintResult",
