@@ -1,3 +1,4 @@
+use chrono::{Datelike, Timelike};
 use ndarray::Array2;
 use pyo3::{prelude::*, types::PyDateTime};
 use sgp4::{parse_2les, Constants};
@@ -8,12 +9,14 @@ use crate::ephemeris::ephemeris_common::{
 };
 use crate::ephemeris::position_velocity::PositionVelocityData;
 use crate::utils::conversions;
+use crate::utils::tle_utils;
 use crate::utils::to_skycoord::AstropyModules;
 
 #[pyclass]
 pub struct TLEEphemeris {
     tle1: String,
     tle2: String,
+    tle_epoch: chrono::DateTime<chrono::Utc>, // TLE epoch timestamp
     teme: Option<Array2<f64>>,
     itrs: Option<Array2<f64>>,
     itrs_skycoord: OnceLock<Py<PyAny>>, // Lazy-initialized cached SkyCoord object for ITRS
@@ -25,23 +28,108 @@ pub struct TLEEphemeris {
 #[pymethods]
 impl TLEEphemeris {
     #[new]
-    #[pyo3(signature = (tle1, tle2, begin, end, step_size=60, *, polar_motion=false))]
+    #[pyo3(signature = (tle1=None, tle2=None, begin=None, end=None, step_size=60, *, polar_motion=false, tle=None, norad_id=None, norad_name=None))]
+    #[allow(clippy::too_many_arguments)]
     fn new(
         _py: Python,
-        tle1: String,
-        tle2: String,
-        begin: &Bound<'_, PyDateTime>,
-        end: &Bound<'_, PyDateTime>,
+        tle1: Option<String>,
+        tle2: Option<String>,
+        begin: Option<&Bound<'_, PyDateTime>>,
+        end: Option<&Bound<'_, PyDateTime>>,
         step_size: i64,
         polar_motion: bool,
+        tle: Option<String>,
+        norad_id: Option<u32>,
+        norad_name: Option<String>,
     ) -> PyResult<Self> {
+        // Determine which method to use for getting TLE data
+        let (line1, line2, tle_epoch) = if let (Some(l1), Some(l2)) = (tle1, tle2) {
+            // Legacy method: tle1 and tle2 parameters
+            let epoch = tle_utils::extract_tle_epoch(&l1).map_err(|e| {
+                pyo3::exceptions::PyValueError::new_err(format!(
+                    "Failed to extract TLE epoch: {}",
+                    e
+                ))
+            })?;
+            (l1, l2, Some(epoch))
+        } else if let Some(tle_param) = tle {
+            // New method: tle parameter (file path or URL)
+            let tle_data = if tle_param.starts_with("http://") || tle_param.starts_with("https://")
+            {
+                // Download from URL with caching
+                tle_utils::download_tle_with_cache(&tle_param).map_err(|e| {
+                    pyo3::exceptions::PyValueError::new_err(format!(
+                        "Failed to download TLE from URL: {}",
+                        e
+                    ))
+                })?
+            } else {
+                // Read from file
+                tle_utils::read_tle_file(&tle_param).map_err(|e| {
+                    pyo3::exceptions::PyValueError::new_err(format!(
+                        "Failed to read TLE from file: {}",
+                        e
+                    ))
+                })?
+            };
+            let epoch = tle_utils::extract_tle_epoch(&tle_data.line1).map_err(|e| {
+                pyo3::exceptions::PyValueError::new_err(format!(
+                    "Failed to extract TLE epoch: {}",
+                    e
+                ))
+            })?;
+            (tle_data.line1, tle_data.line2, Some(epoch))
+        } else if let Some(nid) = norad_id {
+            // Fetch from Celestrak by NORAD ID
+            let tle_data = tle_utils::fetch_tle_by_norad_id(nid).map_err(|e| {
+                pyo3::exceptions::PyValueError::new_err(format!(
+                    "Failed to fetch TLE from Celestrak by NORAD ID: {}",
+                    e
+                ))
+            })?;
+            let epoch = tle_utils::extract_tle_epoch(&tle_data.line1).map_err(|e| {
+                pyo3::exceptions::PyValueError::new_err(format!(
+                    "Failed to extract TLE epoch: {}",
+                    e
+                ))
+            })?;
+            (tle_data.line1, tle_data.line2, Some(epoch))
+        } else if let Some(name) = norad_name {
+            // Fetch from Celestrak by satellite name
+            let tle_data = tle_utils::fetch_tle_by_name(&name).map_err(|e| {
+                pyo3::exceptions::PyValueError::new_err(format!(
+                    "Failed to fetch TLE from Celestrak by name: {}",
+                    e
+                ))
+            })?;
+            let epoch = tle_utils::extract_tle_epoch(&tle_data.line1).map_err(|e| {
+                pyo3::exceptions::PyValueError::new_err(format!(
+                    "Failed to extract TLE epoch: {}",
+                    e
+                ))
+            })?;
+            (tle_data.line1, tle_data.line2, Some(epoch))
+        } else {
+            return Err(pyo3::exceptions::PyValueError::new_err(
+                "Must provide either (tle1, tle2), tle, norad_id, or norad_name parameters",
+            ));
+        };
+
+        // Check that begin and end are provided
+        let begin = begin.ok_or_else(|| {
+            pyo3::exceptions::PyValueError::new_err("begin parameter is required")
+        })?;
+        let end = end
+            .ok_or_else(|| pyo3::exceptions::PyValueError::new_err("end parameter is required"))?;
+
         // Use common timestamp generation logic
         let times = generate_timestamps(begin, end, step_size)?;
 
         // Create the TLEEphemeris object
         let mut ephemeris: TLEEphemeris = TLEEphemeris {
-            tle1,
-            tle2,
+            tle1: line1,
+            tle2: line2,
+            tle_epoch: tle_epoch.unwrap(),
             teme: None,
             itrs: None,
             itrs_skycoord: OnceLock::new(),
@@ -65,6 +153,34 @@ impl TLEEphemeris {
         Ok(ephemeris)
     }
 
+    /// Get the epoch of the TLE as a Python datetime object
+    #[getter]
+    fn tle_epoch(&self, py: Python) -> PyResult<Py<PyAny>> {
+        // Convert chrono::DateTime<Utc> to Python datetime with UTC timezone
+        let epoch = self.tle_epoch;
+
+        let dt = pyo3::types::PyDateTime::new(
+            py,
+            epoch.year(),
+            epoch.month() as u8,
+            epoch.day() as u8,
+            epoch.hour() as u8,
+            epoch.minute() as u8,
+            epoch.second() as u8,
+            epoch.timestamp_subsec_micros(),
+            None,
+        )?;
+
+        // Get UTC timezone and replace
+        let datetime_mod = py.import("datetime")?;
+        let utc_tz = datetime_mod.getattr("timezone")?.getattr("utc")?;
+        let kwargs = pyo3::types::PyDict::new(py);
+        kwargs.set_item("tzinfo", utc_tz)?;
+        let dt_with_tz = dt.call_method("replace", (), Some(&kwargs))?;
+
+        Ok(dt_with_tz.into())
+    }
+
     #[getter]
     fn teme_pv(&self, py: Python) -> Option<Py<PositionVelocityData>> {
         self.teme
@@ -107,12 +223,57 @@ impl TLEEphemeris {
         self.get_gcrs_pv(py)
     }
 
+    #[getter]
+    fn latitude(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
+        <Self as EphemerisBase>::get_latitude(self, py)
+    }
+
+    #[getter]
+    fn latitude_deg(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
+        <Self as EphemerisBase>::get_latitude_deg(self, py)
+    }
+
+    #[getter]
+    fn latitude_rad(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
+        <Self as EphemerisBase>::get_latitude_rad(self, py)
+    }
+
+    #[getter]
+    fn longitude(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
+        <Self as EphemerisBase>::get_longitude(self, py)
+    }
+
+    #[getter]
+    fn longitude_deg(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
+        <Self as EphemerisBase>::get_longitude_deg(self, py)
+    }
+
+    #[getter]
+    fn longitude_rad(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
+        <Self as EphemerisBase>::get_longitude_rad(self, py)
+    }
+
     // Getter for times but call it timestamp and convert to python datetime
     #[getter]
     fn timestamp(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
         self.get_timestamp(py)
     }
 
+    #[getter]
+    fn height(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
+        <Self as EphemerisBase>::get_height(self, py)
+    }
+
+    #[getter]
+    fn height_m(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
+        <Self as EphemerisBase>::get_height_m(self, py)
+    }
+
+    #[getter]
+    fn height_km(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
+        <Self as EphemerisBase>::get_height_km(self, py)
+    }
+
     /// propagate_to_teme() -> np.ndarray
     ///
     /// Propagates the satellite to the times specified during initialization.
