@@ -9,10 +9,13 @@ pub mod config;
 pub mod conversions;
 pub mod eop_cache;
 pub mod eop_provider;
+pub mod geo;
+pub mod interpolation;
 pub mod leap_seconds;
 pub mod math_utils;
 pub mod naif_ids;
 pub mod time_utils;
+pub mod tle_utils;
 pub mod to_skycoord;
 pub mod ut1_provider;
 pub mod vector_math;
